@@ -39,7 +39,7 @@ import org.apache.tomcat.util.buf.ByteChunk;
  * internal collection classes is performed.
  *
  * @author Craig R. McClanahan
- * @version $Id: SavedRequest.java 939336 2010-04-29 15:00:41Z kkolinko $
+ * @version $Id: SavedRequest.java 1043272 2010-12-08 01:31:51Z markt $
  */
 
 public final class SavedRequest {
@@ -54,7 +54,7 @@ public final class SavedRequest {
         cookies.add(cookie);
     }
 
-    public Iterator getCookies() {
+    public Iterator<Cookie> getCookies() {
         return (cookies.iterator());
     }
 
@@ -69,7 +69,7 @@ public final class SavedRequest {
         new HashMap<String,ArrayList<String>>();
 
     public void addHeader(String name, String value) {
-        ArrayList<String> values = (ArrayList<String>) headers.get(name);
+        ArrayList<String> values = headers.get(name);
         if (values == null) {
             values = new ArrayList<String>();
             headers.put(name, values);
@@ -77,14 +77,14 @@ public final class SavedRequest {
         values.add(value);
     }
 
-    public Iterator getHeaderNames() {
+    public Iterator<String> getHeaderNames() {
         return (headers.keySet().iterator());
     }
 
-    public Iterator getHeaderValues(String name) {
-        ArrayList values = (ArrayList) headers.get(name);
+    public Iterator<String> getHeaderValues(String name) {
+        ArrayList<String> values = headers.get(name);
         if (values == null)
-            return ((new ArrayList()).iterator());
+            return ((new ArrayList<String>()).iterator());
         else
             return (values.iterator());
     }
@@ -99,7 +99,7 @@ public final class SavedRequest {
         locales.add(locale);
     }
 
-    public Iterator getLocales() {
+    public Iterator<Locale> getLocales() {
         return (locales.iterator());
     }
 
@@ -118,28 +118,6 @@ public final class SavedRequest {
     }
 
 
-
-    /**
-     * The set of request parameters associated with this Request.  Each
-     * entry is keyed by the parameter name, pointing at a String array of
-     * the corresponding values.
-     */
-    private HashMap<String,String[]> parameters =
-        new HashMap<String,String[]>();
-
-    public void addParameter(String name, String values[]) {
-        parameters.put(name, values);
-    }
-
-    public Iterator getParameterNames() {
-        return (parameters.keySet().iterator());
-    }
-
-    public String[] getParameterValues(String name) {
-        return ((String[]) parameters.get(name));
-    }
-
-
     /**
      * The query string associated with this Request.
      */
