@@ -16,6 +16,7 @@
  */
 package org.apache.catalina.connector;
 
+import java.net.InetAddress;
 import java.util.Arrays;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -42,7 +43,7 @@ import org.apache.tomcat.util.res.StringManager;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Id: Connector.java 1200173 2011-11-10 06:11:10Z kkolinko $
+ * @version $Id: Connector.java 1229558 2012-01-10 14:36:06Z rjung $
  */
 
 
@@ -60,7 +61,6 @@ public class Connector extends LifecycleMBeanBase  {
 
     // ------------------------------------------------------------ Constructor
 
-
     public Connector() {
         this(null);
     }
@@ -121,7 +121,7 @@ public class Connector extends LifecycleMBeanBase  {
     /**
      * The port number on which we listen for requests.
      */
-    protected int port = 0;
+    protected int port = -1;
 
 
     /**
@@ -257,14 +257,12 @@ public class Connector extends LifecycleMBeanBase  {
          replacements.put("acceptCount", "backlog");
          replacements.put("connectionLinger", "soLinger");
          replacements.put("connectionTimeout", "soTimeout");
-         replacements.put("randomFile", "randomfile");
          replacements.put("rootFile", "rootfile");
      }
 
 
     // ------------------------------------------------------------- Properties
 
-
     /**
      * Return a configured property.
      */
@@ -514,7 +512,9 @@ public class Connector extends LifecycleMBeanBase  {
     }
 
     /**
-     * Return the port number on which we listen for requests.
+     * Return the port number on which this connector is configured to listen
+     * for requests. The special value of 0 means select a random free port
+     * when the socket is bound.
      */
     public int getPort() {
 
@@ -536,6 +536,16 @@ public class Connector extends LifecycleMBeanBase  {
     }
 
 
+    /**
+     * Return the port number on which this connector is listening to requests.
+     * If the special value for {@link #port} of zero is used then this method
+     * will report the actual port bound.
+     */
+    public int getLocalPort() {
+        return ((Integer) getProperty("localPort")).intValue();
+    }
+
+
     /**
      * Return the Coyote protocol handler in use.
      */
@@ -881,13 +891,22 @@ public class Connector extends LifecycleMBeanBase  {
         StringBuilder sb = new StringBuilder("type=");
         sb.append(type);
         sb.append(",port=");
-        sb.append(getPort());
-        if (addressObj != null) {
-            String address = addressObj.toString();
-            if (address.length() > 0) {
-                sb.append(",address=");
-                sb.append(ObjectName.quote(address));
-            }
+        int port = getPort();
+        if (port > 0) {
+            sb.append(getPort());
+        } else {
+            sb.append("auto-");
+            sb.append(getProperty("nameIndex"));
+        }
+        String address = "";
+        if (addressObj instanceof InetAddress) {
+            address = ((InetAddress) addressObj).getHostAddress();
+        } else if (addressObj != null) {
+            address = addressObj.toString();
+        }
+        if (address.length() > 0) {
+            sb.append(",address=");
+            sb.append(ObjectName.quote(address));
         }
         return sb.toString();
     }
@@ -955,7 +974,7 @@ public class Connector extends LifecycleMBeanBase  {
     protected void startInternal() throws LifecycleException {
 
         // Validate settings before starting
-        if (getPort() < 1) {
+        if (getPort() < 0) {
             throw new LifecycleException(sm.getString(
                     "coyoteConnector.invalidPort", Integer.valueOf(getPort())));
         }
@@ -1031,11 +1050,18 @@ public class Connector extends LifecycleMBeanBase  {
         StringBuilder sb = new StringBuilder("Connector[");
         sb.append(getProtocol());
         sb.append('-');
-        sb.append(getPort());
+        int port = getPort();
+        if (port > 0) {
+            sb.append(getPort());
+        } else {
+            sb.append("auto-");
+            sb.append(getProperty("nameIndex"));
+        }
         sb.append(']');
         return sb.toString();
     }
 
+
     // -------------------- JMX registration  --------------------
 
     @Override
