@@ -18,16 +18,19 @@
 package org.apache.catalina.ha.tcp;
 
 import java.io.IOException;
-import java.util.StringTokenizer;
-import java.util.regex.Pattern;
 import java.util.ArrayList;
-import java.util.List;
 import java.util.Iterator;
+import java.util.List;
+import java.util.regex.Pattern;
+import java.util.regex.PatternSyntaxException;
+
 import javax.servlet.ServletException;
 
+import org.apache.catalina.Context;
 import org.apache.catalina.Manager;
 import org.apache.catalina.Session;
-import org.apache.catalina.Context;
+import org.apache.catalina.connector.Request;
+import org.apache.catalina.connector.Response;
 import org.apache.catalina.core.StandardContext;
 import org.apache.catalina.ha.CatalinaCluster;
 import org.apache.catalina.ha.ClusterManager;
@@ -36,10 +39,8 @@ import org.apache.catalina.ha.ClusterSession;
 import org.apache.catalina.ha.ClusterValve;
 import org.apache.catalina.ha.session.DeltaManager;
 import org.apache.catalina.ha.session.DeltaSession;
-import org.apache.catalina.connector.Request;
-import org.apache.catalina.connector.Response;
-import org.apache.catalina.util.StringManager;
 import org.apache.catalina.valves.ValveBase;
+import org.apache.tomcat.util.res.StringManager;
 
 /**
  * <p>Implementation of a Valve that logs interesting contents from the
@@ -57,13 +58,13 @@ import org.apache.catalina.valves.ValveBase;
  * @author Craig R. McClanahan
  * @author Filip Hanik
  * @author Peter Rossbach
- * @version $Id: ReplicationValve.java 939514 2010-04-29 23:55:14Z kkolinko $
+ * @version $Id: ReplicationValve.java 1133197 2011-06-07 23:37:29Z markt $
  */
 
 public class ReplicationValve
     extends ValveBase implements ClusterValve {
     
-    private static org.apache.juli.logging.Log log =
+    private static final org.apache.juli.logging.Log log =
         org.apache.juli.logging.LogFactory.getLog( ReplicationValve.class );
 
     // ----------------------------------------------------- Instance Variables
@@ -78,25 +79,21 @@ public class ReplicationValve
     /**
      * The StringManager for this package.
      */
-    protected static StringManager sm =
+    protected static final StringManager sm =
         StringManager.getManager(Constants.Package);
 
     private CatalinaCluster cluster = null ;
 
     /**
-     * holds file endings to not call for like images and others
+     * Filter expression 
      */
-    protected java.util.regex.Pattern[] reqFilters = new java.util.regex.Pattern[0];
-    
-    /**
-     * Orginal filter 
-     */
-    protected String filter ;
+    protected Pattern filter = null;
     
     /**
      * crossContext session container 
      */
-    protected ThreadLocal crossContextSessions = new ThreadLocal() ;
+    protected ThreadLocal<ArrayList<DeltaSession>> crossContextSessions =
+        new ThreadLocal<ArrayList<DeltaSession>>() ;
     
     /**
      * doProcessingStats (default = off)
@@ -124,11 +121,13 @@ public class ReplicationValve
     // ------------------------------------------------------------- Properties
 
     public ReplicationValve() {
+        super(true);
     }
     
     /**
      * Return descriptive information about this Valve implementation.
      */
+    @Override
     public String getInfo() {
 
         return (info);
@@ -138,6 +137,7 @@ public class ReplicationValve
     /**
      * @return Returns the cluster.
      */
+    @Override
     public CatalinaCluster getCluster() {
         return cluster;
     }
@@ -145,6 +145,7 @@ public class ReplicationValve
     /**
      * @param cluster The cluster to set.
      */
+    @Override
     public void setCluster(CatalinaCluster cluster) {
         this.cluster = cluster;
     }
@@ -153,11 +154,14 @@ public class ReplicationValve
      * @return Returns the filter
      */
     public String getFilter() {
-       return filter ;
+       if (filter == null) {
+           return null;
+       }
+       return filter.toString();
     }
 
     /**
-     * compile filter string to regular expressions
+     * compile filter string to regular expression
      * @see Pattern#compile(java.lang.String)
      * @param filter
      *            The filter to set.
@@ -165,19 +169,15 @@ public class ReplicationValve
     public void setFilter(String filter) {
         if (log.isDebugEnabled())
             log.debug(sm.getString("ReplicationValve.filter.loading", filter));
-        this.filter = filter;
-        StringTokenizer t = new StringTokenizer(filter, ";");
-        this.reqFilters = new Pattern[t.countTokens()];
-        int i = 0;
-        while (t.hasMoreTokens()) {
-            String s = t.nextToken();
-            if (log.isTraceEnabled())
-                log.trace(sm.getString("ReplicationValve.filter.token", s));
+        
+        if (filter == null || filter.length() == 0) {
+            this.filter = null;
+        } else {
             try {
-                reqFilters[i++] = Pattern.compile(s);
-            } catch (Exception x) {
-                log.error(sm.getString("ReplicationValve.filter.token.failure",
-                        s), x);
+                this.filter = Pattern.compile(filter);
+            } catch (PatternSyntaxException pse) {
+                log.error(sm.getString("ReplicationValve.filter.failure",
+                        filter), pse);
             }
         }
     }
@@ -274,21 +274,6 @@ public class ReplicationValve
         return totalSendTime;
     }
 
-    /**
-     * @return Returns the reqFilters.
-     */
-    protected java.util.regex.Pattern[] getReqFilters() {
-        return reqFilters;
-    }
-    
-    /**
-     * @param reqFilters The reqFilters to set.
-     */
-    protected void setReqFilters(java.util.regex.Pattern[] reqFilters) {
-        this.reqFilters = reqFilters;
-    }
-    
-    
     // --------------------------------------------------------- Public Methods
     
     /**
@@ -299,7 +284,7 @@ public class ReplicationValve
      * @param session cross context session
      */
     public void registerReplicationSession(DeltaSession session) {
-        List sessions = (List)crossContextSessions.get();
+        List<DeltaSession> sessions = crossContextSessions.get();
         if(sessions != null) {
             if(!sessions.contains(session)) {
                 if(log.isDebugEnabled())
@@ -321,6 +306,7 @@ public class ReplicationValve
      * @exception IOException if an input/output error occurs
      * @exception ServletException if a servlet error occurs
      */
+    @Override
     public void invoke(Request request, Response response)
         throws IOException, ServletException
     {
@@ -342,7 +328,7 @@ public class ReplicationValve
                 if(log.isDebugEnabled())
                     log.debug(sm.getString("ReplicationValve.crossContext.add"));
                 //FIXME add Pool of Arraylists
-                crossContextSessions.set(new ArrayList());
+                crossContextSessions.set(new ArrayList<DeltaSession>());
             }
             getNext().invoke(request, response);
             if(context != null) {
@@ -381,7 +367,7 @@ public class ReplicationValve
 
     
     /**
-     * reset the active statitics 
+     * reset the active statistics 
      */
     public void resetStatistics() {
         totalRequestTime = 0 ;
@@ -393,18 +379,6 @@ public class ReplicationValve
         nrOfCrossContextSendRequests = 0;
     }
     
-    /**
-     * Return a String rendering of this object.
-     */
-    public String toString() {
-
-        StringBuffer sb = new StringBuffer("ReplicationValve[");
-        if (container != null)
-            sb.append(container.getName());
-        sb.append("]");
-        return (sb.toString());
-
-    }
 
     // --------------------------------------------------------- Protected Methods
 
@@ -446,11 +420,10 @@ public class ReplicationValve
      * @param containerCluster
      */
     protected void sendCrossContextSession(CatalinaCluster containerCluster) {
-        Object sessions = crossContextSessions.get();
-        if(sessions != null && sessions instanceof List
-                && ((List)sessions).size() >0) {
-            for(Iterator iter = ((List)sessions).iterator(); iter.hasNext() ;) {          
-                Session session = (Session)iter.next();
+        List<DeltaSession> sessions = crossContextSessions.get();
+        if(sessions != null && sessions.size() >0) {
+            for(Iterator<DeltaSession> iter = sessions.iterator(); iter.hasNext() ;) {          
+                Session session = iter.next();
                 if(log.isDebugEnabled())
                     log.debug(sm.getString("ReplicationValve.crossContext.sendDelta",  
                             session.getManager().getContainer().getName() ));
@@ -464,22 +437,21 @@ public class ReplicationValve
   
     /**
      * Fix memory leak for long sessions with many changes, when no backup member exists!
-     * @param request current request after responce is generated
+     * @param request current request after response is generated
      * @param isCrossContext check crosscontext threadlocal
      */
     protected void resetReplicationRequest(Request request, boolean isCrossContext) {
         Session contextSession = request.getSessionInternal(false);
-        if(contextSession != null && contextSession instanceof DeltaSession){
+        if(contextSession instanceof DeltaSession){
             resetDeltaRequest(contextSession);
             ((DeltaSession)contextSession).setPrimarySession(true);
         }
         if(isCrossContext) {
-            Object sessions = crossContextSessions.get();
-            if(sessions != null && sessions instanceof List
-               && ((List)sessions).size() >0) {
-                Iterator iter = ((List)sessions).iterator();
+            List<DeltaSession> sessions = crossContextSessions.get();
+            if(sessions != null && sessions.size() >0) {
+                Iterator<DeltaSession> iter = sessions.iterator();
                 for(; iter.hasNext() ;) {          
-                    Session session = (Session)iter.next();
+                    Session session = iter.next();
                     resetDeltaRequest(session);
                     if(session instanceof DeltaSession)
                         ((DeltaSession)contextSession).setPrimarySession(true);
@@ -526,7 +498,7 @@ public class ReplicationValve
 
    /**
     * Send message delta message from request session 
-    * @param request current request
+    * @param session current session
     * @param manager session manager
     * @param cluster replication cluster
     */
@@ -549,11 +521,7 @@ public class ReplicationValve
     protected void send(ClusterManager manager, CatalinaCluster cluster, String sessionId) {
         ClusterMessage msg = manager.requestCompleted(sessionId);
         if (msg != null) {
-            if(manager.doDomainReplication()) {
-                cluster.sendClusterDomain(msg);
-            } else {
-                cluster.send(msg);
-            }
+            cluster.send(msg);
             if(doStatistics())
                 nrOfSendRequests++;
         }
@@ -583,22 +551,18 @@ public class ReplicationValve
      * @return True if no session change
      */
     protected boolean isRequestWithoutSessionChange(String uri) {
-
-        boolean filterfound = false;
-
-        for (int i = 0; (i < reqFilters.length) && (!filterfound); i++) {
-            java.util.regex.Matcher matcher = reqFilters[i].matcher(uri);
-            filterfound = matcher.matches();
-        }
-        return filterfound;
+        Pattern f = filter;
+        return f != null && f.matcher(uri).matches();
     }
 
     /**
-     * protocol cluster replications stats
+     * Protocol cluster replications stats
      * @param requestTime
      * @param clusterTime
      */
     protected  void updateStats(long requestTime, long clusterTime) {
+        // TODO: Async requests may trigger multiple replication requests. How,
+        //       if at all, should the stats handle this?
         synchronized(this) {
             lastSendTime=System.currentTimeMillis();
             totalSendTime+=lastSendTime - clusterTime;
@@ -609,14 +573,14 @@ public class ReplicationValve
             if ( (nrOfRequests % 100) == 0 ) {
                  log.info(sm.getString("ReplicationValve.stats",
                      new Object[]{
-                         new Long(totalRequestTime/nrOfRequests),
-                         new Long(totalSendTime/nrOfRequests),
-                         new Long(nrOfRequests),
-                         new Long(nrOfSendRequests),
-                         new Long(nrOfCrossContextSendRequests),
-                         new Long(nrOfFilterRequests),
-                         new Long(totalRequestTime),
-                         new Long(totalSendTime)}));
+                         Long.valueOf(totalRequestTime/nrOfRequests),
+                         Long.valueOf(totalSendTime/nrOfRequests),
+                         Long.valueOf(nrOfRequests),
+                         Long.valueOf(nrOfSendRequests),
+                         Long.valueOf(nrOfCrossContextSendRequests),
+                         Long.valueOf(nrOfFilterRequests),
+                         Long.valueOf(totalRequestTime),
+                         Long.valueOf(totalSendTime)}));
              }
         }
     }
@@ -636,13 +600,12 @@ public class ReplicationValve
             Session session = manager.findSession(id);
             if (session instanceof ClusterSession) {
                 ClusterSession cses = (ClusterSession) session;
-                if (cses != null) {
-                    if (log.isDebugEnabled())
-                        log.debug(sm.getString(
-                                "ReplicationValve.session.indicator", request.getContext().getName(),id,
-                                primaryIndicatorName, cses.isPrimarySession()));
-                    request.setAttribute(primaryIndicatorName, cses.isPrimarySession()?Boolean.TRUE:Boolean.FALSE);
-                }
+                if (log.isDebugEnabled())
+                    log.debug(sm.getString(
+                            "ReplicationValve.session.indicator", request.getContext().getName(),id,
+                            primaryIndicatorName,
+                            Boolean.valueOf(cses.isPrimarySession())));
+                request.setAttribute(primaryIndicatorName, cses.isPrimarySession()?Boolean.TRUE:Boolean.FALSE);
             } else {
                 if (log.isDebugEnabled()) {
                     if (session != null) {
