@@ -67,7 +67,7 @@ import org.xml.sax.SAXParseException;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Id: Catalina.java 1239489 2012-02-02 08:47:02Z markt $
+ * @version $Id: Catalina.java 1302650 2012-03-19 20:48:49Z markt $
  */
 public class Catalina {
 
@@ -180,7 +180,10 @@ public class Catalina {
     }
 
     public ClassLoader getParentClassLoader() {
-        return parentClassLoader;
+        if (parentClassLoader != null) {
+            return (parentClassLoader);
+        }
+        return ClassLoader.getSystemClassLoader();
     }
 
     public void setServer(Server server) {
@@ -448,17 +451,25 @@ public class Catalina {
             Digester digester = createStopDigester();
             digester.setClassLoader(Thread.currentThread().getContextClassLoader());
             File file = configFile();
+            FileInputStream fis = null;
             try {
                 InputSource is =
                     new InputSource("file://" + file.getAbsolutePath());
-                FileInputStream fis = new FileInputStream(file);
+                fis = new FileInputStream(file);
                 is.setByteStream(fis);
                 digester.push(this);
                 digester.parse(is);
-                fis.close();
             } catch (Exception e) {
                 log.error("Catalina.stop: ", e);
                 System.exit(1);
+            } finally {
+                if (fis != null) {
+                    try {
+                        fis.close();
+                    } catch (IOException e) {
+                        // Ignore
+                    }
+                }
             }
         } else {
             // Server object already present. Must be running as a service
@@ -472,26 +483,40 @@ public class Catalina {
 
         // Stop the existing server
         s = getServer();
-        try {
-            if (s.getPort()>0) {
-                Socket socket = new Socket(s.getAddress(), s.getPort());
-                OutputStream stream = socket.getOutputStream();
+        if (s.getPort()>0) {
+            Socket socket = null;
+            OutputStream stream = null;
+            try {
+                socket = new Socket(s.getAddress(), s.getPort());
+                stream = socket.getOutputStream();
                 String shutdown = s.getShutdown();
                 for (int i = 0; i < shutdown.length(); i++) {
                     stream.write(shutdown.charAt(i));
                 }
                 stream.flush();
-                stream.close();
-                socket.close();
-            } else {
-                log.error(sm.getString("catalina.stopServer"));
+            } catch (IOException e) {
+                log.error("Catalina.stop: ", e);
                 System.exit(1);
+            } finally {
+                if (stream != null) {
+                    try {
+                        stream.close();
+                    } catch (IOException e) {
+                        // Ignore
+                    }
+                }
+                if (socket != null) {
+                    try {
+                        socket.close();
+                    } catch (IOException e) {
+                        // Ignore
+                    }
+                }
             }
-        } catch (IOException e) {
-            log.error("Catalina.stop: ", e);
+        } else {
+            log.error(sm.getString("catalina.stopServer"));
             System.exit(1);
         }
-
     }
 
 
@@ -574,7 +599,6 @@ public class Catalina {
             inputSource.setByteStream(inputStream);
             digester.push(this);
             digester.parse(inputSource);
-            inputStream.close();
         } catch (SAXParseException spe) {
             log.warn("Catalina.start using " + getConfigFile() + ": " +
                     spe.getMessage());
@@ -582,6 +606,12 @@ public class Catalina {
         } catch (Exception e) {
             log.warn("Catalina.start using " + getConfigFile() + ": " , e);
             return;
+        } finally {
+            try {
+                inputStream.close();
+            } catch (IOException e) {
+                // Ignore
+            }
         }
 
         getServer().setCatalina(this);
