@@ -16,6 +16,7 @@
  */
 package org.apache.coyote;
 
+import java.io.IOException;
 import java.net.InetAddress;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentLinkedQueue;
@@ -28,6 +29,8 @@ import javax.management.MBeanServer;
 import javax.management.MalformedObjectNameException;
 import javax.management.ObjectName;
 
+import org.apache.coyote.http11.upgrade.UpgradeInbound;
+import org.apache.coyote.http11.upgrade.UpgradeProcessor;
 import org.apache.juli.logging.Log;
 import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.modeler.Registry;
@@ -47,6 +50,13 @@ public abstract class AbstractProtocol implements ProtocolHandler,
         StringManager.getManager(Constants.Package);
 
 
+    /**
+     * Counter used to generate unique JMX names for connectors using automatic
+     * port binding.
+     */
+    private static final AtomicInteger nameCounter = new AtomicInteger(0);
+
+
     /**
      * Name of MBean for the Global Request Processor.
      */
@@ -59,6 +69,14 @@ public abstract class AbstractProtocol implements ProtocolHandler,
     protected ObjectName tpOname = null;
 
     
+    /**
+     * Unique ID for this connector. Only used if the connector is configured
+     * to use a random port as the port will change if stop(), start() is
+     * called.
+     */
+    private int nameIndex = 0;
+
+
     /**
      * Endpoint that provides low-level network I/O - must be matched to the
      * ProtocolHandler implementation (ProtocolHandler using BIO, requires BIO
@@ -194,6 +212,8 @@ public abstract class AbstractProtocol implements ProtocolHandler,
     }
 
 
+    public int getLocalPort() { return endpoint.getLocalPort(); }
+
     /*
      * When Tomcat expects data from the client, this is the time Tomcat will
      * wait for that data to arrive before closing the connection.
@@ -220,6 +240,15 @@ public abstract class AbstractProtocol implements ProtocolHandler,
 
     // ---------------------------------------------------------- Public methods
 
+    public synchronized int getNameIndex() {
+        if (nameIndex == 0) {
+            nameIndex = nameCounter.incrementAndGet();
+        }
+
+        return nameIndex;
+    }
+
+
     /**
      * The name will be prefix-address-port if address is non-null and
      * prefix-port if the address is null. The name will be appropriately quoted
@@ -229,10 +258,22 @@ public abstract class AbstractProtocol implements ProtocolHandler,
         StringBuilder name = new StringBuilder(getNamePrefix());
         name.append('-');
         if (getAddress() != null) {
-            name.append(getAddress());
+            name.append(getAddress().getHostAddress());
             name.append('-');
         }
-        name.append(endpoint.getPort());
+        int port = getPort();
+        if (port == 0) {
+            // Auto binding is in use. Check if port is known
+            name.append("auto-");
+            name.append(getNameIndex());
+            port = getLocalPort();
+            if (port != -1) {
+                name.append('-');
+                name.append(port);
+            }
+        } else {
+            name.append(port);
+        }
         return ObjectName.quote(name.toString());
     }
 
@@ -313,15 +354,22 @@ public abstract class AbstractProtocol implements ProtocolHandler,
 
         StringBuilder name = new StringBuilder(getDomain());
         name.append(":type=ProtocolHandler,port=");
-        name.append(getPort());
+        int port = getPort();
+        if (port > 0) {
+            name.append(getPort());
+        } else {
+            name.append("auto-");
+            name.append(getNameIndex());
+        }
         InetAddress address = getAddress();
         if (address != null) {
             name.append(",address=");
-            name.append(ObjectName.quote(address.toString()));
+            name.append(ObjectName.quote(address.getHostAddress()));
         }
         return new ObjectName(name.toString());
     }
 
+
     // ------------------------------------------------------- Lifecycle methods
 
     /*
@@ -460,7 +508,7 @@ public abstract class AbstractProtocol implements ProtocolHandler,
     
     // ------------------------------------------- Connection handler base class
     
-    protected abstract static class AbstractConnectionHandler<S,P extends AbstractProcessor<S>>
+    protected abstract static class AbstractConnectionHandler<S,P extends Processor<S>>
             implements AbstractEndpoint.Handler {
 
         protected abstract Log getLog();
@@ -468,8 +516,8 @@ public abstract class AbstractProtocol implements ProtocolHandler,
         protected RequestGroupInfo global = new RequestGroupInfo();
         protected AtomicLong registerCount = new AtomicLong(0);
 
-        protected ConcurrentHashMap<S,P> connections =
-            new ConcurrentHashMap<S,P>();
+        protected ConcurrentHashMap<S,Processor<S>> connections =
+            new ConcurrentHashMap<S,Processor<S>>();
 
         protected RecycledProcessors<P,S> recycledProcessors =
             new RecycledProcessors<P,S>(this);
@@ -491,13 +539,7 @@ public abstract class AbstractProtocol implements ProtocolHandler,
 
         public SocketState process(SocketWrapper<S> socket,
                 SocketStatus status) {
-            P processor = connections.remove(socket.getSocket());
-
-            if (getLog().isDebugEnabled()) {
-                getLog().debug("process() entry " +
-                        "Socket: [" + logHashcode(socket.getSocket()) + "], " +
-                        "Processor [" + logHashcode(processor) + "]");
-            }
+            Processor<S> processor = connections.remove(socket.getSocket());
 
             socket.setAsync(false);
 
@@ -509,53 +551,35 @@ public abstract class AbstractProtocol implements ProtocolHandler,
                     processor = createProcessor();
                 }
 
-                if (getLog().isDebugEnabled()) {
-                    getLog().debug("process() gotProcessor " +
-                            "Socket: [" + logHashcode(socket.getSocket()) + "], " +
-                            "Processor [" + logHashcode(processor) + "]");
-                }
-
                 initSsl(socket, processor);
 
                 SocketState state = SocketState.CLOSED;
                 do {
                     if (processor.isAsync() || state == SocketState.ASYNC_END) {
                         state = processor.asyncDispatch(status);
-                        if (getLog().isDebugEnabled()) {
-                            getLog().debug("process() asyncDispatch " +
-                                    "Socket: [" + logHashcode(socket.getSocket()) + "], " +
-                                    "Processor: [" + logHashcode(processor) + "], " +
-                                    "State: [" + state.toString() + "]");
-                        }
                     } else if (processor.isComet()) {
                         state = processor.event(status);
-                        if (getLog().isDebugEnabled()) {
-                            getLog().debug("process() event " +
-                                    "Socket: [" + logHashcode(socket.getSocket()) + "], " +
-                                    "Processor: [" + logHashcode(processor) + "], " +
-                                    "State: [" + state.toString() + "]");
-                        }
+                    } else if (processor.isUpgrade()) {
+                        state = processor.upgradeDispatch();
                     } else {
                         state = processor.process(socket);
-                        if (getLog().isDebugEnabled()) {
-                            getLog().debug("process() process " +
-                                    "Socket: [" + logHashcode(socket.getSocket()) + "], " +
-                                    "Processor: [" + logHashcode(processor) + "], " +
-                                    "State: [" + state.toString() + "]");
-                        }
                     }
     
                     if (state != SocketState.CLOSED && processor.isAsync()) {
                         state = processor.asyncPostProcess();
-                        if (getLog().isDebugEnabled()) {
-                            getLog().debug("process() asyncPostProcess " +
-                                    "Socket: [" + logHashcode(socket.getSocket()) + "], " +
-                                    "Processor: [" + logHashcode(processor) + "], " +
-                                    "State: [" + state.toString() + "]");
-                        }
-
                     }
-                } while (state == SocketState.ASYNC_END);
+
+                    if (state == SocketState.UPGRADING) {
+                        // Get the UpgradeInbound handler
+                        UpgradeInbound inbound = processor.getUpgradeInbound();
+                        // Release the Http11 processor to be re-used
+                        release(socket, processor, false, false);
+                        // Create the light-weight upgrade processor
+                        processor = createUpgradeProcessor(socket, inbound);
+                        inbound.onUpgradeComplete();
+                    }
+                } while (state == SocketState.ASYNC_END ||
+                        state == SocketState.UPGRADING);
 
                 if (state == SocketState.LONG) {
                     // In the middle of processing a request/response. Keep the
@@ -571,9 +595,14 @@ public abstract class AbstractProtocol implements ProtocolHandler,
                     // closed. If it works, the socket will be re-added to the
                     // poller
                     release(socket, processor, false, false);
+                } else if (state == SocketState.UPGRADED) {
+                    // Need to keep the connection associated with the processor
+                    longPoll(socket, processor);
                 } else {
                     // Connection closed. OK to recycle the processor.
-                    release(socket, processor, true, false);
+                    if (!(processor instanceof UpgradeProcessor)) {
+                        release(socket, processor, true, false);
+                    }
                 }
                 return state;
             } catch(java.net.SocketException e) {
@@ -595,24 +624,24 @@ public abstract class AbstractProtocol implements ProtocolHandler,
                 // less-than-verbose logs.
                 getLog().error(sm.getString("ajpprotocol.proto.error"), e);
             }
-            release(socket, processor, true, false);
+            // Don't try to add upgrade processors back into the pool
+            if (!(processor instanceof UpgradeProcessor)) {
+                release(socket, processor, true, false);
+            }
             return SocketState.CLOSED;
         }
         
-        private String logHashcode (Object o) {
-            if (o == null) {
-                return "null";
-            } else {
-                return Integer.toString(o.hashCode());
-            }
-        }
-
         protected abstract P createProcessor();
-        protected abstract void initSsl(SocketWrapper<S> socket, P processor);
-        protected abstract void longPoll(SocketWrapper<S> socket, P processor);
-        protected abstract void release(SocketWrapper<S> socket, P processor,
-                boolean socketClosing, boolean addToPoller);
-
+        protected abstract void initSsl(SocketWrapper<S> socket,
+                Processor<S> processor);
+        protected abstract void longPoll(SocketWrapper<S> socket,
+                Processor<S> processor);
+        protected abstract void release(SocketWrapper<S> socket,
+                Processor<S> processor, boolean socketClosing,
+                boolean addToPoller);
+        protected abstract Processor<S> createUpgradeProcessor(
+                SocketWrapper<S> socket,
+                UpgradeInbound inbound) throws IOException;
 
         protected void register(AbstractProcessor<S> processor) {
             if (getProtocol().getDomain() != null) {
@@ -641,12 +670,16 @@ public abstract class AbstractProtocol implements ProtocolHandler,
             }
         }
 
-        protected void unregister(AbstractProcessor<S> processor) {
+        protected void unregister(Processor<S> processor) {
             if (getProtocol().getDomain() != null) {
                 synchronized (this) {
                     try {
-                        RequestInfo rp =
-                            processor.getRequest().getRequestProcessor();
+                        Request r = processor.getRequest();
+                        if (r == null) {
+                            // Probably an UpgradeProcessor
+                            return;
+                        }
+                        RequestInfo rp = r.getRequestProcessor();
                         rp.setGlobalProcessor(null);
                         ObjectName rpName = rp.getRpName();
                         if (getLog().isDebugEnabled()) {
@@ -663,8 +696,8 @@ public abstract class AbstractProtocol implements ProtocolHandler,
         }
     }
     
-    protected static class RecycledProcessors<P extends AbstractProcessor<S>, S>
-            extends ConcurrentLinkedQueue<P> {
+    protected static class RecycledProcessors<P extends Processor<S>, S>
+            extends ConcurrentLinkedQueue<Processor<S>> {
 
         private static final long serialVersionUID = 1L;
         private transient AbstractConnectionHandler<S,P> handler;
@@ -675,7 +708,7 @@ public abstract class AbstractProtocol implements ProtocolHandler,
         }
 
         @Override
-        public boolean offer(P processor) {
+        public boolean offer(Processor<S> processor) {
             int cacheSize = handler.getProtocol().getProcessorCache();
             boolean offer = cacheSize == -1 ? true : size.get() < cacheSize;
             //avoid over growing our cache or add after we have stopped
@@ -691,8 +724,8 @@ public abstract class AbstractProtocol implements ProtocolHandler,
         }
     
         @Override
-        public P poll() {
-            P result = super.poll();
+        public Processor<S> poll() {
+            Processor<S> result = super.poll();
             if (result != null) {
                 size.decrementAndGet();
             }
@@ -701,7 +734,7 @@ public abstract class AbstractProtocol implements ProtocolHandler,
     
         @Override
         public void clear() {
-            P next = poll();
+            Processor<S> next = poll();
             while (next != null) {
                 handler.unregister(next);
                 next = poll();
