@@ -16,7 +16,12 @@
  */
 package org.apache.coyote.http11;
 
+import java.io.IOException;
+
 import org.apache.coyote.AbstractProtocol;
+import org.apache.coyote.Processor;
+import org.apache.coyote.http11.upgrade.UpgradeAprProcessor;
+import org.apache.coyote.http11.upgrade.UpgradeInbound;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.util.net.AbstractEndpoint;
@@ -211,9 +216,9 @@ public class Http11AprProtocol extends AbstractHttp11Protocol {
          */
         @Override
         public void release(SocketWrapper<Long> socket,
-                Http11AprProcessor processor, boolean isSocketClosing,
+                Processor<Long> processor, boolean isSocketClosing,
                 boolean addToPoller) {
-            processor.recycle();
+            processor.recycle(isSocketClosing);
             recycledProcessors.offer(processor);
             if (addToPoller && proto.endpoint.isRunning()) {
                 ((AprEndpoint)proto.endpoint).getPoller().add(
@@ -223,20 +228,26 @@ public class Http11AprProtocol extends AbstractHttp11Protocol {
 
         @Override
         protected void initSsl(SocketWrapper<Long> socket,
-                Http11AprProcessor processor) {
+                Processor<Long> processor) {
             // NOOP for APR
         }
 
         @Override
         protected void longPoll(SocketWrapper<Long> socket,
-                Http11AprProcessor processor) {
+                Processor<Long> processor) {
             connections.put(socket.getSocket(), processor);
 
             if (processor.isAsync()) {
+                // Async
                 socket.setAsync(true);
-            } else if (processor.comet && proto.endpoint.isRunning()) {
+            } else if (processor.isComet() && proto.endpoint.isRunning()) {
+                // Comet
                 ((AprEndpoint) proto.endpoint).getCometPoller().add(
                         socket.getSocket().longValue(), false);
+            } else {
+                // Upgraded
+                ((AprEndpoint) proto.endpoint).getPoller().add(
+                        socket.getSocket().longValue(), false);
             }
         }
 
@@ -263,5 +274,12 @@ public class Http11AprProtocol extends AbstractHttp11Protocol {
             register(processor);
             return processor;
         }
+
+        @Override
+        protected Processor<Long> createUpgradeProcessor(
+                SocketWrapper<Long> socket, UpgradeInbound inbound)
+                throws IOException {
+            return new UpgradeAprProcessor(socket, inbound);
+        }
     }
 }
