@@ -126,7 +126,8 @@ public class Http11NioProcessor extends AbstractHttp11Processor<NioChannel> {
                 if (attach != null) {
                     attach.setComet(comet);
                     if (comet) {
-                        Integer comettimeout = (Integer) request.getAttribute("org.apache.tomcat.comet.timeout");
+                        Integer comettimeout = (Integer) request.getAttribute(
+                                org.apache.coyote.Constants.COMET_TIMEOUT_ATTR);
                         if (comettimeout != null) {
                             attach.setTimeout(comettimeout.longValue());
                         }
@@ -261,7 +262,8 @@ public class Http11NioProcessor extends AbstractHttp11Processor<NioChannel> {
             if (attach != null)  {
                 attach.setComet(comet);
                 if (comet) {
-                    Integer comettimeout = (Integer) request.getAttribute("org.apache.tomcat.comet.timeout");
+                    Integer comettimeout = (Integer) request.getAttribute(
+                            org.apache.coyote.Constants.COMET_TIMEOUT_ATTR);
                     if (comettimeout != null) {
                         attach.setTimeout(comettimeout.longValue());
                     }
@@ -510,7 +512,7 @@ public class Http11NioProcessor extends AbstractHttp11Processor<NioChannel> {
     @Override
     protected boolean prepareSendfile(OutputFilter[] outputFilters) {
         String fileName = (String) request.getAttribute(
-                "org.apache.tomcat.sendfile.filename");
+                org.apache.coyote.Constants.SENDFILE_FILENAME_ATTR);
         if (fileName != null) {
             // No entity body sent here
             outputBuffer.addActiveFilter(outputFilters[Constants.VOID_FILTER]);
@@ -518,9 +520,9 @@ public class Http11NioProcessor extends AbstractHttp11Processor<NioChannel> {
             sendfileData = new NioEndpoint.SendfileData();
             sendfileData.fileName = fileName;
             sendfileData.pos = ((Long) request.getAttribute(
-                    "org.apache.tomcat.sendfile.start")).longValue();
+                    org.apache.coyote.Constants.SENDFILE_FILE_START_ATTR)).longValue();
             sendfileData.length = ((Long) request.getAttribute(
-                    "org.apache.tomcat.sendfile.end")).longValue() - sendfileData.pos;
+                    org.apache.coyote.Constants.SENDFILE_FILE_END_ATTR)).longValue() - sendfileData.pos;
             return true;
         }
         return false;
@@ -544,6 +546,7 @@ public class Http11NioProcessor extends AbstractHttp11Processor<NioChannel> {
     /**
      * Set the SSL information for this HTTP connection.
      */
+    @Override
     public void setSslSupport(SSLSupport sslSupport) {
         this.sslSupport = sslSupport;
     }
