@@ -171,7 +171,7 @@ public class JspC implements Options {
     protected boolean smapSuppressed = true;
     protected boolean smapDumped = false;
     protected boolean caching = true;
-    protected Map<String, TagLibraryInfo> cache =
+    protected final Map<String, TagLibraryInfo> cache =
         new HashMap<String, TagLibraryInfo>();
 
     protected String compiler = null;
@@ -196,7 +196,7 @@ public class JspC implements Options {
     /**
      * The pages.
      */
-    protected List<String> pages = new Vector<String>();
+    protected final List<String> pages = new Vector<String>();
 
     /**
      * Needs better documentation, this data member does.
@@ -1300,19 +1300,19 @@ public class JspC implements Options {
                     Localizer.getMessage("jsp.error.jspc.no_uriroot"));
             }
 
-            if( context==null ) {
+            File uriRootF = new File(uriRoot);
+            if (!uriRootF.isDirectory()) {
+                throw new JasperException(
+                    Localizer.getMessage("jsp.error.jspc.uriroot_not_dir"));
+            }
+
+            if(context == null) {
                 initServletContext();
             }
 
             // No explicit pages, we'll process all .jsp in the webapp
             if (pages.size() == 0) {
-                scanFiles( new File( uriRoot ));
-            }
-
-            File uriRootF = new File(uriRoot);
-            if (!uriRootF.exists() || !uriRootF.isDirectory()) {
-                throw new JasperException(
-                    Localizer.getMessage("jsp.error.jspc.uriroot_not_dir"));
+                scanFiles(uriRootF);
             }
 
             initWebXml();
