@@ -718,6 +718,9 @@ public class NioEndpoint extends AbstractEndpoint {
     public boolean processSocket(NioChannel socket, SocketStatus status, boolean dispatch) {
         try {
             KeyAttachment attachment = (KeyAttachment)socket.getAttachment(false);
+            if (attachment == null) {
+                return false;
+            }
             attachment.setCometNotify(false); //will get reset upon next reg
             SocketProcessor sc = processorCache.poll();
             if ( sc == null ) sc = new SocketProcessor(socket,status);
@@ -960,7 +963,6 @@ public class NioEndpoint extends AbstractEndpoint {
             // exit, otherwise parallel closure of sockets which are still
             // in the poller can cause problems
             close = true;
-            events.clear();
             selector.wakeup();
         }
 
@@ -995,6 +997,9 @@ public class NioEndpoint extends AbstractEndpoint {
             if ( r==null) r = new PollerEvent(socket,null,interestOps);
             else r.reset(socket,null,interestOps);
             addEvent(r);
+            if (close) {
+                processSocket(socket, SocketStatus.STOP, false);
+            }
         }
 
         /**
@@ -1111,12 +1116,21 @@ public class NioEndpoint extends AbstractEndpoint {
                         }
                     }
 
-                    boolean hasEvents = events();
+                    boolean hasEvents = false;
 
                     // Time to terminate?
                     if (close) {
+                        events();
                         timeout(0, false);
+                        try {
+                            selector.close();
+                        } catch (IOException ioe) {
+                            log.error(sm.getString(
+                                    "endpoint.nio.selectorCloseFail"), ioe);
+                        }
                         break;
+                    } else {
+                        hasEvents = events();
                     }
                     try {
                         if ( !close ) {
@@ -1130,8 +1144,14 @@ public class NioEndpoint extends AbstractEndpoint {
                             wakeupCounter.set(0);
                         }
                         if (close) {
+                            events();
                             timeout(0, false);
-                            selector.close();
+                            try {
+                                selector.close();
+                            } catch (IOException ioe) {
+                                log.error(sm.getString(
+                                        "endpoint.nio.selectorCloseFail"), ioe);
+                            }
                             break;
                         }
                     } catch ( NullPointerException x ) {
@@ -1255,7 +1275,7 @@ public class NioEndpoint extends AbstractEndpoint {
         public boolean processSendfile(SelectionKey sk, KeyAttachment attachment, boolean reg, boolean event) {
             NioChannel sc = null;
             try {
-                //unreg(sk,attachment);//only do this if we do process send file on a separate thread
+                unreg(sk, attachment, sk.readyOps());
                 SendfileData sd = attachment.getSendfileData();
                 if ( sd.fchannel == null ) {
                     File f = new File(sd.fileName);
@@ -1615,7 +1635,7 @@ public class NioEndpoint extends AbstractEndpoint {
                                     if (ka!=null) ka.setComet(false);
                                     socket.getPoller().cancelledKey(key, SocketStatus.ERROR, false);
                                 }
-                                if (socket!=null) nioChannels.offer(socket);
+                                nioChannels.offer(socket);
                                 socket = null;
                                 if ( ka!=null ) keyCache.offer(ka);
                                 ka = null;
@@ -1629,7 +1649,7 @@ public class NioEndpoint extends AbstractEndpoint {
                             ka = (KeyAttachment) key.attachment();
                             socket.getPoller().cancelledKey(key, SocketStatus.DISCONNECT, false);
                         }
-                        if (socket!=null) nioChannels.offer(socket);
+                        nioChannels.offer(socket);
                         socket = null;
                         if ( ka!=null ) keyCache.offer(ka);
                         ka = null;
