@@ -36,6 +36,7 @@ import javax.servlet.http.HttpServletResponse;
 
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
 import org.junit.Test;
@@ -938,6 +939,13 @@ public class TestAsyncContextImpl extends TomcatBaseTest {
             if (flush) {
                 resp.flushBuffer();
             }
+            try {
+                // Give the original thread a chance to exit the
+                // ErrorReportValve before we throw this exception
+                Thread.sleep(500);
+            } catch (InterruptedException e) {
+                throw new ServletException(e);
+            }
             throw new ServletException("Opps.");
         }
     }
@@ -1168,7 +1176,16 @@ public class TestAsyncContextImpl extends TomcatBaseTest {
     }
 
     @Test
-    public void testBug51197() throws Exception {
+    public void testBug51197a() throws Exception {
+        doTestBug51197(false);
+    }
+
+    @Test
+    public void testBug51197b() throws Exception {
+        doTestBug51197(true);
+    }
+
+    private void doTestBug51197(boolean threaded) throws Exception {
         // Setup Tomcat instance
         Tomcat tomcat = getTomcatInstance();
 
@@ -1178,7 +1195,7 @@ public class TestAsyncContextImpl extends TomcatBaseTest {
         Context ctx = tomcat.addContext("", docBase.getAbsolutePath());
 
         AsyncErrorServlet asyncErrorServlet =
-            new AsyncErrorServlet(HttpServletResponse.SC_BAD_REQUEST);
+            new AsyncErrorServlet(HttpServletResponse.SC_BAD_REQUEST, threaded);
         Wrapper wrapper =
             Tomcat.addServlet(ctx, "asyncErrorServlet", asyncErrorServlet);
         wrapper.setAsyncSupported(true);
@@ -1194,28 +1211,41 @@ public class TestAsyncContextImpl extends TomcatBaseTest {
         url.append(getPort());
         url.append("/asyncErrorServlet");
 
-        int rc = getUrl(url.toString(), new ByteChunk(), null);
+        ByteChunk res = new ByteChunk();
+        int rc = getUrl(url.toString(), res, null);
 
         assertEquals(HttpServletResponse.SC_BAD_REQUEST, rc);
 
+        // SRV 10.9.2 - Writing the response is entirely the application's
+        // responsibility when an error occurs on an application thread.
+        // The test servlet writes no content in this case.
+        if (threaded) {
+            assertEquals(0, res.getLength());
+        } else {
+            assertTrue(res.getLength() > 0);
+        }
+
         // Without this test may complete before access log has a chance to log
         // the request
         Thread.sleep(REQUEST_TIME);
 
         // Check the access log
-        alv.validateAccessLog(1, HttpServletResponse.SC_BAD_REQUEST, TIMEOUT,
-                TIMEOUT + TIMEOUT_MARGIN + REQUEST_TIME);
-
+        alv.validateAccessLog(1, HttpServletResponse.SC_BAD_REQUEST, 0,
+                REQUEST_TIME);
     }
 
     private static class AsyncErrorServlet extends HttpServlet {
 
         private static final long serialVersionUID = 1L;
 
-        private int status = 200;
+        public static final String ERROR_MESSAGE = "It broke.";
 
-        public AsyncErrorServlet(int status) {
+        private int status;
+        private boolean threaded;
+
+        public AsyncErrorServlet(int status, boolean threaded) {
             this.status = status;
+            this.threaded = threaded;
         }
 
         @Override
@@ -1224,18 +1254,25 @@ public class TestAsyncContextImpl extends TomcatBaseTest {
 
             final AsyncContext actxt = req.startAsync();
             actxt.setTimeout(TIMEOUT);
-            actxt.start(new Runnable() {
-                @Override
-                public void run() {
-                    try {
-                        ((HttpServletResponse) actxt.getResponse()).sendError(
-                                status);
-                    } catch (IOException e) {
-                        // Ignore
+            if (threaded) {
+                actxt.start(new Runnable() {
+                    @Override
+                    public void run() {
+                        try {
+                            HttpServletResponse resp =
+                                    (HttpServletResponse) actxt.getResponse();
+                            resp.sendError(status, ERROR_MESSAGE);
+                            // Complete so there is no delay waiting for the
+                            // timeout
+                            actxt.complete();
+                        } catch (IOException e) {
+                            // Ignore
+                        }
                     }
-                }
-            });
+                });
+            } else {
+                resp.sendError(status);
+            }
         }
     }
-
 }
